@@ -1,9 +1,13 @@
+import asyncio
+import inspect
 import logging
 import socket
 import ssl
 import sys
+from typing import List, Tuple
 
 from uvicorn.importer import ImportFromStringError, import_from_string
+from uvicorn.middleware.asgi2 import ASGI2Middleware
 from uvicorn.middleware.debug import DebugMiddleware
 from uvicorn.middleware.message_logger import MessageLoggerMiddleware
 from uvicorn.middleware.proxy_headers import ProxyHeadersMiddleware
@@ -28,7 +32,7 @@ WS_PROTOCOLS = {
     "wsproto": "uvicorn.protocols.websockets.wsproto_impl:WSProtocol",
 }
 LIFESPAN = {
-    "auto": "uvicorn.lifespan.auto:LifespanAuto",
+    "auto": "uvicorn.lifespan.on:LifespanOn",
     "on": "uvicorn.lifespan.on:LifespanOn",
     "off": "uvicorn.lifespan.off:LifespanOff",
 }
@@ -37,6 +41,10 @@ LOOP_SETUPS = {
     "asyncio": "uvicorn.loops.asyncio:asyncio_setup",
     "uvloop": "uvicorn.loops.uvloop:uvloop_setup",
 }
+INTERFACES = ["auto", "asgi3", "asgi2", "wsgi"]
+
+# Fallback to 'ssl.PROTOCOL_SSLv23' in order to support Python < 3.5.3.
+SSL_PROTOCOL_VERSION = getattr(ssl, "PROTOCOL_TLS", ssl.PROTOCOL_SSLv23)
 
 
 def get_logger(log_level):
@@ -74,7 +82,7 @@ class Config:
         log_level="info",
         logger=None,
         access_log=True,
-        wsgi=False,
+        interface="auto",
         debug=False,
         reload=False,
         reload_dirs=None,
@@ -88,10 +96,11 @@ class Config:
         callback_notify=None,
         ssl_keyfile=None,
         ssl_certfile=None,
-        ssl_version=ssl.PROTOCOL_TLS,
+        ssl_version=SSL_PROTOCOL_VERSION,
         ssl_cert_reqs=ssl.CERT_NONE,
         ssl_ca_certs=None,
         ssl_ciphers="TLSv1",
+        headers=None,
     ):
         self.app = app
         self.host = host
@@ -105,7 +114,7 @@ class Config:
         self.log_level = log_level
         self.logger = logger
         self.access_log = access_log
-        self.wsgi = wsgi
+        self.interface = interface
         self.debug = debug
         self.reload = reload
         self.workers = workers
@@ -122,6 +131,8 @@ class Config:
         self.ssl_cert_reqs = ssl_cert_reqs
         self.ssl_ca_certs = ssl_ca_certs
         self.ssl_ciphers = ssl_ciphers
+        self.headers = headers if headers else []  # type: List[str]
+        self.encoded_headers = None  # type: List[Tuple[bytes, bytes]]
 
         if self.ssl_keyfile or self.ssl_certfile:
             self.ssl = create_ssl_context(
@@ -145,6 +156,16 @@ class Config:
     def load(self):
         assert not self.loaded
 
+        encoded_headers = [
+            (key.lower().encode("latin1"), value.encode("latin1"))
+            for key, value in self.headers
+        ]
+        self.encoded_headers = (
+            encoded_headers
+            if b"server" in dict(encoded_headers)
+            else [(b"server", b"uvicorn")] + encoded_headers
+        )  # type: List[Tuple[bytes, bytes]]
+
         if isinstance(self.http, str):
             self.http_protocol_class = import_from_string(HTTP_PROTOCOLS[self.http])
         else:
@@ -163,9 +184,22 @@ class Config:
             self.logger_instance.error("Error loading ASGI app. %s" % exc)
             sys.exit(1)
 
-        if self.wsgi:
+        if self.interface == "auto":
+            if inspect.isclass(self.loaded_app):
+                use_asgi_3 = hasattr(self.loaded_app, "__await__")
+            elif inspect.isfunction(self.loaded_app):
+                use_asgi_3 = asyncio.iscoroutinefunction(self.loaded_app)
+            else:
+                call = getattr(self.loaded_app, "__call__", None)
+                use_asgi_3 = asyncio.iscoroutinefunction(call)
+            self.interface = "asgi3" if use_asgi_3 else "asgi2"
+
+        if self.interface == "wsgi":
             self.loaded_app = WSGIMiddleware(self.loaded_app)
             self.ws_protocol_class = None
+        elif self.interface == "asgi2":
+            self.loaded_app = ASGI2Middleware(self.loaded_app)
+
         if self.debug:
             self.loaded_app = DebugMiddleware(self.loaded_app)
         if self.logger_instance.level <= logging.DEBUG:
