@@ -12,9 +12,11 @@ import click
 
 from uvicorn.config import (
     HTTP_PROTOCOLS,
+    INTERFACES,
     LIFESPAN,
     LOG_LEVELS,
     LOOP_SETUPS,
+    SSL_PROTOCOL_VERSION,
     WS_PROTOCOLS,
     Config,
     get_logger,
@@ -26,6 +28,7 @@ HTTP_CHOICES = click.Choice(HTTP_PROTOCOLS.keys())
 WS_CHOICES = click.Choice(WS_PROTOCOLS.keys())
 LIFESPAN_CHOICES = click.Choice(LIFESPAN.keys())
 LOOP_CHOICES = click.Choice(LOOP_SETUPS.keys())
+INTERFACE_CHOICES = click.Choice(INTERFACES)
 
 HANDLED_SIGNALS = (
     signal.SIGINT,  # Unix signal 2. Sent by Ctrl+C.
@@ -98,10 +101,11 @@ HANDLED_SIGNALS = (
     show_default=True,
 )
 @click.option(
-    "--wsgi",
-    is_flag=True,
-    default=False,
-    help="Use WSGI as the application interface, instead of ASGI.",
+    "--interface",
+    type=INTERFACE_CHOICES,
+    default="auto",
+    help="Select ASGI3, ASGI2, or WSGI as the application interface.",
+    show_default=True,
 )
 @click.option(
     "--log-level",
@@ -157,7 +161,7 @@ HANDLED_SIGNALS = (
 @click.option(
     "--ssl-version",
     type=int,
-    default=ssl.PROTOCOL_TLS,
+    default=SSL_PROTOCOL_VERSION,
     help="SSL version to use (see stdlib ssl module's)",
     show_default=True,
 )
@@ -182,6 +186,12 @@ HANDLED_SIGNALS = (
     help="Ciphers to use (see stdlib ssl module's)",
     show_default=True,
 )
+@click.option(
+    "--header",
+    "headers",
+    multiple=True,
+    help="Specify custom default HTTP response headers as a Name:Value pair",
+)
 def main(
     app,
     host: str,
@@ -192,7 +202,7 @@ def main(
     http: str,
     ws: str,
     lifespan: str,
-    wsgi: bool,
+    interface: str,
     debug: bool,
     reload: bool,
     reload_dirs: typing.List[str],
@@ -210,6 +220,7 @@ def main(
     ssl_cert_reqs: int,
     ssl_ca_certs: str,
     ssl_ciphers: str,
+    headers: typing.List[str],
 ):
     sys.path.insert(0, ".")
 
@@ -225,7 +236,7 @@ def main(
         "lifespan": lifespan,
         "log_level": log_level,
         "access_log": not no_access_log,
-        "wsgi": wsgi,
+        "interface": interface,
         "debug": debug,
         "reload": reload,
         "reload_dirs": reload_dirs if reload_dirs else None,
@@ -241,6 +252,7 @@ def main(
         "ssl_cert_reqs": ssl_cert_reqs,
         "ssl_ca_certs": ssl_ca_certs,
         "ssl_ciphers": ssl_ciphers,
+        "headers": list([header.split(":") for header in headers]),
     }
     run(**kwargs)
 
@@ -286,6 +298,7 @@ class Server:
         self.started = False
         self.should_exit = False
         self.force_exit = False
+        self.last_notified = 0
 
     def run(self, sockets=None, shutdown_servers=True):
         self.config.setup_event_loop()
@@ -306,6 +319,8 @@ class Server:
 
         self.logger.info("Started server process [{}]".format(process_id))
         await self.startup(sockets=sockets)
+        if self.should_exit:
+            return
         await self.main_loop()
         await self.shutdown(shutdown_servers=shutdown_servers)
         self.logger.info("Finished server process [{}]".format(process_id))
@@ -314,6 +329,9 @@ class Server:
         config = self.config
 
         await self.lifespan.startup()
+        if self.lifespan.should_exit:
+            self.should_exit = True
+            return
 
         create_protocol = functools.partial(
             config.http_protocol_class, config=config, server_state=self.server_state
@@ -343,7 +361,11 @@ class Server:
 
         elif config.uds is not None:
             # Create a socket using UNIX domain socket.
+            uds_perms = 0o666
+            if os.path.exists(config.uds):
+                uds_perms = os.stat(config.uds).st_mode
             server = await loop.create_unix_server(create_protocol, path=config.uds)
+            os.chmod(config.uds, uds_perms)
             message = "Uvicorn running on unix socket %s (Press CTRL+C to quit)"
             self.logger.info(message % config.uds)
             self.servers = [server]
@@ -375,14 +397,14 @@ class Server:
             current_time = time.time()
             current_date = formatdate(current_time, usegmt=True).encode()
             self.server_state.default_headers = [
-                (b"server", b"uvicorn"),
-                (b"date", current_date),
-            ]
+                (b"date", current_date)
+            ] + self.config.encoded_headers
 
-        # Callback to `callback_notify` once every `timeout_notify` seconds.
-        if self.config.callback_notify is not None:
-            if counter % (10 * self.config.timeout_notify) == 0:
-                await self.config.callback_notify()
+            # Callback to `callback_notify` once every `timeout_notify` seconds.
+            if self.config.callback_notify is not None:
+                if current_time - self.last_notified > self.config.timeout_notify:
+                    self.last_notified = current_time
+                    await self.config.callback_notify()
 
         # Determine if we should exit.
         if self.should_exit:
