@@ -54,22 +54,18 @@ class FlowControl:
             self._is_writable_event.set()
 
 
-class ServiceUnavailable:
-    def __init__(self, scope):
-        pass
-
-    async def __call__(self, receive, send):
-        await send(
-            {
-                "type": "http.response.start",
-                "status": 503,
-                "headers": [
-                    (b"content-type", b"text/plain; charset=utf-8"),
-                    (b"connection", b"close"),
-                ],
-            }
-        )
-        await send({"type": "http.response.body", "body": b"Service Unavailable"})
+async def service_unavailable(scope, receive, send):
+    await send(
+        {
+            "type": "http.response.start",
+            "status": 503,
+            "headers": [
+                (b"content-type", b"text/plain; charset=utf-8"),
+                (b"connection", b"close"),
+            ],
+        }
+    )
+    await send({"type": "http.response.body", "body": b"Service Unavailable"})
 
 
 class H11Protocol(asyncio.Protocol):
@@ -140,6 +136,7 @@ class H11Protocol(asyncio.Protocol):
                 pass
 
         self.message_event.set()
+        self.flow.resume_writing()
 
     def eof_received(self):
         pass
@@ -203,7 +200,7 @@ class H11Protocol(asyncio.Protocol):
                     len(self.connections) >= self.limit_concurrency
                     or len(self.tasks) >= self.limit_concurrency
                 ):
-                    app = ServiceUnavailable
+                    app = service_unavailable
                     message = "Exceeded concurrency limit."
                     self.logger.warning(message)
                 else:
@@ -369,8 +366,7 @@ class RequestResponseCycle:
     # ASGI exception wrapper
     async def run_asgi(self, app):
         try:
-            asgi = app(self.scope)
-            result = await asgi(self.receive, self.send)
+            result = await app(self.scope, self.receive, self.send)
         except BaseException as exc:
             msg = "Exception in ASGI application\n"
             self.logger.error(msg, exc_info=exc)
@@ -413,12 +409,12 @@ class RequestResponseCycle:
     async def send(self, message):
         message_type = message["type"]
 
+        if self.flow.write_paused and not self.disconnected:
+            await self.flow.drain()
+
         if self.disconnected:
             return
 
-        if self.flow.write_paused:
-            await self.flow.drain()
-
         if not self.response_started:
             # Sending response status line and headers
             if message_type != "http.response.start":
