@@ -143,8 +143,7 @@ class WebSocketProtocol(websockets.WebSocketServerProtocol):
         termination states.
         """
         try:
-            asgi = self.app(self.scope)
-            result = await asgi(self.asgi_receive, self.asgi_send)
+            result = await self.app(self.scope, self.asgi_receive, self.asgi_send)
         except BaseException as exc:
             self.closed_event.set()
             msg = "Exception in ASGI application\n"
@@ -228,9 +227,11 @@ class WebSocketProtocol(websockets.WebSocketServerProtocol):
         except websockets.ConnectionClosed as exc:
             return {"type": "websocket.disconnect", "code": exc.code}
 
-        is_text = isinstance(data, str)
-        return {
-            "type": "websocket.receive",
-            "text": data if is_text else None,
-            "bytes": None if is_text else data,
-        }
+        msg = {"type": "websocket.receive"}
+
+        if isinstance(data, str):
+            msg["text"] = data
+        else:
+            msg["bytes"] = data
+
+        return msg
